@@ -7,7 +7,7 @@ use warnings;
 use Helios::Job;
 use Helios::Config;
 
-our $VERSION = '2.71_4460';
+our $VERSION = '2.71_4770';
 
 # CHANGES:
 # 2012-01-22: Minor changes to comments.
@@ -16,6 +16,8 @@ our $VERSION = '2.71_4460';
 # Replaced all try {} with eval {}.  Added --verbose option to return jobid.
 # [LH] [2013-11-01]: changed cmd line arg parsing so -v and -n work as well as
 # --verbose and --no-validate
+# [LH] [2013-11-24]: Added new --verbose and jobtype documentation to POD.  
+# Removed old code already commented out.
 
 =head1 NAME
 
@@ -23,7 +25,7 @@ helios_job_submit.pl - Submit a job to the Helios job processing system from the
 
 =head1 SYNOPSIS
 
- helios_job_submit.pl [--jobid] [--no-validate] I<jobclass> [I<jobargs>]
+ helios_job_submit.pl [--verbose] [--no-validate] I<jobclass> [I<jobargs>]
 
  helios_job_submit.pl MyService "<job>params><myarg1>myvalue1</myarg1></params></job>"
 
@@ -31,10 +33,37 @@ helios_job_submit.pl - Submit a job to the Helios job processing system from the
 
 =head1 DESCRIPTION
 
-Use helios_job_submit.pl to submit a job to the Helios job processing system from the cmd line.  
-The first parameter is the service class, and the second is the parameter XML that will be passed to 
-the worker class for the job.  If the second parameter isn't given, the program will accept input 
-from STDIN.
+Use helios_job_submit.pl to submit a job to the Helios job processing system 
+from the command line or a shell script.  In the simplest form, 2 options are 
+required.  The first parameter is the jobtype (usually the name of your 
+service), and the second is the XML string representing the job's arguments.  
+If the second parameter is not given, helios_job_submit.pl will read the job
+argument string from STDIN.  If the job submission is successful, the command 
+will exit with an exitstatus of 0.  If unsuccessful, the command will exit 
+with a non-zero status and will print the error to STDERR.
+
+In addition to the normal command options, there are 2 optional parameters 
+that can be specified.  These must be specified I<before> the jobtype and 
+argument string:
+
+=over 4
+
+=item --verbose
+
+If --verbose is specified, the jobid of the new job is returned via STDOUT.  
+Without --verbose, the jobid is not returned; this is a backward compatibility 
+measure because Helios pre-2.80 did not return a jobid.
+
+=item --no-validate
+
+By default, Helios job argument strings are specified with an XML-like syntax.  
+The helios_job_submit.pl command attempts to insure your job argument string 
+is well-formed XML by default, but this can slow down the job submission 
+process.  Specifying the --no-validate option will turn off the validation step 
+and will speed up the submission process.  Keep in mind your job argument 
+XML will not be validated before submission.
+
+=back
 
 =cut
 
@@ -73,17 +102,6 @@ if ( !defined($JOB_CLASS) || ($JOB_CLASS eq '--help') || ($JOB_CLASS eq '-h') )
 	Pod::Usage::pod2usage(-verbose => 2, -exitstatus => 0);
 }
 
-=old
-#[]
-# instantiate the base worker class just to get the [global] INI params
-# (we need to know where the Helios db is)
-my $WORKER = new Helios::Service;
-# BEGIN [LH] 2012-08-04: Changed old service init to use new Helios::Config API
-$WORKER->prep();
-my $config = $WORKER->getConfig();
-# END [LH] 2013-08-04
-=cut
-
 # BEGIN CODE Copyright (C) 2013 by Logical Helion, LLC.
 my $config = Helios::Config->parseConfig();
 # END CODE Copyright (C) 2013 by Logical Helion, LLC.
@@ -103,16 +121,6 @@ if ( !defined($PARAMS) ) {
 # test the args before we submit
 if ($VALIDATE) { validateParamsXML($PARAMS) or exit(1); }
 
-=old
-#[]
-# create a Helios::Job object and submit it
-my $hjob = Helios::Job->new();
-$hjob->setConfig($config);
-$hjob->setFuncname($JOB_CLASS);
-$hjob->setArgXML($PARAMS);
-my $jobid = $hjob->submit();
-=cut
-
 my $j = Helios::Job->new();
 $j->setConfig($config);
 $j->setJobType($JOB_CLASS);
@@ -142,17 +150,6 @@ if it isn't.
 
 sub validateParamsXML {
 	my $arg = shift;
-=old
-#[]
-#	try {
-#		my $arg = Helios::Job->parseArgXML($arg);
-#		return 1;
-#	} catch Helios::Error::InvalidArg with {
-#		my $e = shift;
-#		print STDERR "Invalid job arguments: $arg (",$e->text(),")\n";
-#		return undef;
-#	};
-=cut
 	
 	# [LH] [2013-10-28]: Replaced all try {} with eval {}	
 	my $valid = 0;
